' ------------------------------------------------------------------------------------
' irr::core::matrix4 wrappers
' ------------------------------------------------------------------------------------

Public

' Constructor Flags
'enum eConstructor
Const EM4CONST_NOTHING:Int = 0
Const EM4CONST_COPY:Int = 1
Const EM4CONST_IDENTITY:Int = 2
Const EM4CONST_TRANSPOSED:Int = 3
Const EM4CONST_INVERSE:Int = 4
Const EM4CONST_INVERSE_TRANSPOSED:Int = 5

Type Matrix4 Extends IrrBase Final

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------

	Method elementAt:Float(row:Int,column:Int)
		Return IrrCore_Matrix4_elementAt(handle,row,column)
	EndMethod
	Method Eq:Matrix4(other:Matrix4)
		IrrCore_Matrix4_Eq(handle,other.handle)
		Return Self
	EndMethod
	Method EqEq:Int(other:Matrix4)
		Return IrrCore_Matrix4_EqEq(handle,other.handle)
	EndMethod
	Method NotEq:Int(other:Matrix4)
		Return IrrCore_Matrix4_NotEq(handle,other.handle)
	EndMethod
	Method MultEq:Matrix4(other:Matrix4)
		IrrCore_Matrix4_MultEq(handle,other.handle)
		Return Self
	EndMethod
	Method Mult:Matrix4(other:Matrix4)
		Return createFromHandle(IrrCore_Matrix4_Mult(handle,other.handle))
	EndMethod	
	Method Idx:Float Ptr(index:Int)
		Return IrrCore_Matrix4_Idx(handle,index)
	EndMethod
	Method EqScalar:Matrix4(scalar:Float)
		IrrCore_Matrix4_EqScalar(handle,scalar)
		Return Self
	EndMethod
	Method MultEqScalar:Matrix4(scalar:Float)
		IrrCore_Matrix4_MultEqScalar(handle,scalar)
		Return Self
	EndMethod
	Method MultScalar:Matrix4(scalar:Float)
		Return Matrix4.createFromHandle(IrrCore_Matrix4_MultScalar(handle,scalar),True)
	EndMethod
	Method PlusEq:Matrix4(other:Matrix4)
		IrrCore_Matrix4_PlusEq(handle,other.handle)
		Return Self
	EndMethod
	Method Plus:Matrix4(other:Matrix4)
		Return Matrix4.createFromHandle(IrrCore_Matrix4_Plus(handle,other.handle),True)
	EndMethod
	Method MinusEq:Matrix4(other:Matrix4)
		IrrCore_Matrix4_MinusEq(handle,other.handle)
		Return Self
	EndMethod
	Method Minus:Matrix4(other:Matrix4)
		Return Matrix4.createFromHandle(IrrCore_Matrix4_Minus(handle,other.handle),True)
	EndMethod
	
	'--------------------------------------------
	' get/manipulation routines
	'--------------------------------------------

	Method buildProjectionMatrixPerspectiveFovRH(fieldOfViewRadians:Float,aspectRatio:Float,zNear:Float,zFar:Float)
		IrrCore_Matrix4_buildProjectionMatrixPerspectiveFovRH(handle,fieldOfViewRadians,aspectRatio,zNear,zFar)
	EndMethod
	Method buildProjectionMatrixPerspectiveFovLH(fieldOfViewRadians:Float,aspectRatio:Float,zNear:Float,zFar:Float)
		IrrCore_Matrix4_buildProjectionMatrixPerspectiveFovLH(handle,fieldOfViewRadians,aspectRatio,zNear,zFar)
	EndMethod
	Method buildProjectionMatrixPerspectiveRH(widthOfViewVolume:Float,heightOfViewVolume:Float,zNear:Float,zFar:Float)
		IrrCore_Matrix4_buildProjectionMatrixPerspectiveRH(handle,widthOfViewVolume,heightOfViewVolume,zNear,zFar)
	EndMethod
	Method buildProjectionMatrixPerspectiveLH(widthOfViewVolume:Float,heightOfViewVolume:Float,zNear:Float,zFar:Float)
		IrrCore_Matrix4_buildProjectionMatrixPerspectiveLH(handle,widthOfViewVolume,heightOfViewVolume,zNear,zFar)
	EndMethod
	Method buildProjectionMatrixOrthoRH(widthOfViewVolume:Float,heightOfViewVolume:Float,zNear:Float,zFar:Float)
		IrrCore_Matrix4_buildProjectionMatrixOrthoRH(handle,widthOfViewVolume,heightOfViewVolume,zNear,zFar)
	EndMethod
	Method buildProjectionMatrixOrthoLH(widthOfViewVolume:Float,heightOfViewVolume:Float,zNear:Float,zFar:Float)
		IrrCore_Matrix4_buildProjectionMatrixOrthoLH(handle,widthOfViewVolume,heightOfViewVolume,zNear,zFar)
	EndMethod
	Method buildCameraLookAtMatrixRH(pPosition:Vector3df,pTarget:Vector3df,pUpVector:Vector3df)
		IrrCore_Matrix4_buildCameraLookAtMatrixRH(handle,pPosition.handle,pTarget.handle,pUpVector.handle)
	EndMethod
	Method buildCameraLookAtMatrixLH(pPosition:Vector3df,pTarget:Vector3df,pUpVector:Vector3df)
		IrrCore_Matrix4_buildCameraLookAtMatrixLH(handle,pPosition.handle,pTarget.handle,pUpVector.handle)
	EndMethod		
	Method buildShadowMatrix(pLight:Vector3df,pPlane:Plane3df,point:Float=1.0)
		IrrCore_Matrix4_buildShadowMatrix(handle,pLight.handle,pPlane.handle,point)
	EndMethod

	Method buildNDCToDCMatrix(area:Recti,zScale:Float)
		IrrCore_Matrix4_buildNDCToDCMatrix(handle,area.handle,zScale)
	EndMethod
	
	Method multiplyWith1x4Matrix(mat:Float Ptr) ' pointer to array of 4 floats representing a 1x4 matrix
		IrrCore_Matrix4_multiplyWith1x4Matrix(handle,mat)
	EndMethod
	
	Method makeInverse:Int()
		Return IrrCore_Matrix4_makeInverse(handle)
	EndMethod
	Method getInverse:Int(pOut:Matrix4)
		Return IrrCore_Matrix4_getInverse(handle,pOut.handle)
	EndMethod
	Method interpolate:Matrix4(pB:Matrix4,time:Float)
		Return createFromHandle(IrrCore_Matrix4_interpolate(handle,pB.handle,time))
	EndMethod
	Method getTransposed:Matrix4()
		Return createFromHandle(IrrCore_Matrix4_getTransposed(handle))
	EndMethod
	
	Method setInverseRotationRadians(pRotation:Vector3df)
		IrrCore_Matrix4_setInverseRotationRadians(handle,pRotation.handle)
	EndMethod
	Method setInverseRotationDegrees(pRotation:Vector3df)
		IrrCore_Matrix4_setInverseRotationDegrees(handle,pRotation.handle)
	EndMethod
	Method SetScale(pScale:Vector3df)
		IrrCore_Matrix4_setScale(handle,pScale.handle)
	EndMethod
	Method inverseTranslateVect(pVect:Vector3df)
		IrrCore_Matrix4_inverseTranslateVect(handle,pVect.handle)
	EndMethod
	Method inverseRotateVect(pVect:Vector3df)
		IrrCore_Matrix4_inverseRotateVect(handle,pVect.handle)
	EndMethod
	Method rotateVect(pVect:Vector3df)
		IrrCore_Matrix4_rotateVect(handle,pVect.handle)
	EndMethod
	Method rotateVectToVect(out:Vector3df, in:Vector3df)
		IrrCore_Matrix4_rotateVectToVect(handle,out.handle,in.handle)
	EndMethod
	Method rotateVectToFloatArray(out:Float[], in:Vector3df) ' Float[] must at least be Float[3]
		IrrCore_Matrix4_rotateVectToFloatArray(handle, out, in.handle)
	EndMethod
	Method transformVectIO(pIn:Vector3df,pOut:Vector3df)
		IrrCore_Matrix4_transformVectIO(handle,pIn.handle,pOut.handle)
	EndMethod
	Method transformVect(pVect:Vector3df)
		IrrCore_Matrix4_transformVect(handle,pVect.handle)
	EndMethod
	Method transformVectArray(out:Float Ptr,in:Vector3df)
		IrrCore_Matrix4_transformVectArray(handle,out,in.handle)
	EndMethod
	Method translateVect(pVect:Vector3df)
		IrrCore_Matrix4_translateVect(handle,pVect.handle)
	EndMethod
	Method transformPlaneIO(pIn:Plane3df,pOut:Plane3df)
		IrrCore_Matrix4_transformPlaneIO(handle,pIn.handle,pOut.handle)
	EndMethod
	Method transformPlane(pPlane:Plane3df)
		IrrCore_Matrix4_transformPlane(handle,pPlane.handle)
	EndMethod
	Method transformBox(pBox:AABbox3df)
		IrrCore_Matrix4_transformBox(handle,pBox.handle)
	EndMethod
	Method transformBoxEx(pBox:AABbox3df)
		IrrCore_Matrix4_transformBoxEx(handle,pBox.handle)
	EndMethod
		
	Method makeIdentity()
		IrrCore_Matrix4_makeIdentity(handle)
	EndMethod
	Method isIdentity:Int()
		Return IrrCore_Matrix4_isIdentity(handle)
	EndMethod
	Method setTranslation(pTranslation:Vector3df)
		IrrCore_Matrix4_setTranslation(handle,pTranslation.handle)
	EndMethod
	Method getTranslation:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Matrix4_getTranslation(handle))
	EndMethod
	Method setInverseTranslation(pTranslation:Vector3df)
		IrrCore_Matrix4_setInverseTranslation(handle,pTranslation.handle)
	EndMethod
	Method setRotationRadians(pRotation:Vector3df)
		IrrCore_Matrix4_setRotationRadians(handle,pRotation.handle)
	EndMethod
	Method setRotationDegrees(pRotation:Vector3df)
		IrrCore_Matrix4_setRotationDegrees(handle,pRotation.handle)
	EndMethod
	Method getRotationDegrees:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Matrix4_getRotationDegrees(handle))
	EndMethod

	Method getScale:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Matrix4_getScale(handle))
	EndMethod

	Method pointer:Float Ptr()
		Return IrrCore_Matrix4_pointer(handle)
	EndMethod

	Method setbyproduct_nocheck(other_a:Matrix4,other_b:Matrix4)
		IrrCore_Matrix4_setbyproduct_nocheck(handle,other_a.handle,other_b.handle)
	EndMethod
	Method setbyproduct(other_a:Matrix4,other_b:Matrix4)
		IrrCore_Matrix4_setbyproduct(handle,other_a.handle,other_b.handle)
	EndMethod
	Method isIdentity_integer_base:Int()
		Return IrrCore_Matrix4_isIdentity_integer_base(handle)
	EndMethod
	Method rotateVectToOut(out:Float Var,in:Vector3df)
		IrrCore_Matrix4_rotateVectToOut(handle,Varptr(out),in.handle)
	EndMethod
	Method transformPlane_new(plane:Plane3df)
		IrrCore_Matrix4_transformPlane_new(handle,plane.handle)
	EndMethod
	Method getInversePrimitive:Int(out:Matrix4)
		Return IrrCore_Matrix4_getInversePrimitive(handle,out.handle)
	EndMethod
	Method getTransposedToDest(dest:Matrix4)
		IrrCore_Matrix4_getTransposedToDest(handle,dest.handle)
	EndMethod
	Method setTextureRotationCenter(rotateRad:Float)
		IrrCore_Matrix4_setTextureRotationCenter(handle,rotateRad)
	EndMethod
	Method setTextureTranslate(x:Float,y:Float)
		IrrCore_Matrix4_setTextureTranslate(handle,x,y)
	EndMethod
	Method setTextureScale(sx:Float,sy:Float)
		IrrCore_Matrix4_setTextureScale(handle,sx,sy)
	EndMethod
	Method setTextureScaleCenter(sx:Float,sy:Float)
		IrrCore_Matrix4_setTextureScaleCenter(handle,sx,sy)
	EndMethod
	Method buildTextureTransform(rotateRad:Float,rotatecenter:Vector2df,translate:Vector2df,scale:Vector2df)
		IrrCore_Matrix4_buildTextureTransform(handle,rotateRad:Float,rotatecenter.handle,translate.handle,scale.handle)
	EndMethod

	Method setM(data:Float[])
		IrrCore_Matrix4_setM(handle,data)
	EndMethod
	
	Method setDefinitelyIdentityMatrix(bIsDefinitelyIdentityMatrix:Int)
		IrrCore_Matrix4_setDefinitelyIdentityMatrix(handle,bIsDefinitelyIdentityMatrix)
	EndMethod
	
	Method getDefinitelyIdentityMatrix:Int()
		Return IrrCore_Matrix4_getDefinitelyIdentityMatrix(handle)
	EndMethod
	
	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function create:Matrix4(eConstructor:Int=EM4CONST_IDENTITY)
		Return createFromHandle(IrrCore_Matrix4_new(eConstructor),True)
	EndFunction

	Function createFromMatrix4:Matrix4(other:Matrix4,eConstructor:Int=EM4CONST_COPY)
		Return createFromHandle(IrrCore_Matrix4_newFromMatrix4(other.handle,eConstructor),True)
	EndFunction

	Function createFromHandle:Matrix4(pMat:Int,bMustDel:Int=True)
		If Not pMat Then Return Null
		Local oRetVal:Matrix4=New Matrix4
		oRetVal.attach(pMat,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Matrix4_del(handle)
	EndMethod	
	
EndType