' ------------------------------------------------------------------------------------
' irr::core::vector3df wrappers
' ------------------------------------------------------------------------------------

Public

Type Vector3df Extends IrrBase Final

	'--------------------------------------------
	' misc routines
	'--------------------------------------------

	Method isBetweenPoints:Int(pBeginVect:Vector3df,pEndVect:Vector3df)
		Return IrrCore_Vector3df_isBetweenPoints(handle,pBeginVect.handle,pEndVect.handle) 
	EndMethod

	Method invert()
		IrrCore_Vector3df_invert(handle)
	EndMethod

	Method setLength(newlength:Float)
		IrrCore_Vector3df_setLength(handle,newlength)
	EndMethod

	Method crossProduct:Vector3df(pOtherVect:Vector3df)
		Return createFromHandle(IrrCore_Vector3df_crossProduct(handle,pOtherVect.handle))
	EndMethod

	Method getInterpolated:Vector3df(pOtherVect:Vector3df,d:Float)
		Return createFromHandle(IrrCore_Vector3df_getInterpolated(handle,pOtherVect.handle,d))
	EndMethod

	Method rotateXZBy(degrees:Double,pCenter:Vector3df)
		IrrCore_Vector3df_rotateXZBy(handle,degrees,pCenter.handle)
	EndMethod

	Method rotateXYBy(degrees:Double,pCenter:Vector3df)
		IrrCore_Vector3df_rotateXYBy(handle,degrees,pCenter.handle)
	EndMethod

	Method rotateYZBy(degrees:Double,pCenter:Vector3df)
		IrrCore_Vector3df_rotateYZBy(handle,degrees,pCenter.handle)
	EndMethod

	Method normalize:Vector3df()
		Return createFromHandle(IrrCore_Vector3df_normalize(handle),False)
	EndMethod
	
	Method getDistanceFrom:Float(pOther:Vector3df)
		Return IrrCore_Vector3df_getDistanceFrom(handle,pOther.handle)
	EndMethod

	Method getDistanceFromSQ:Float(pOther:Vector3df)
		Return IrrCore_Vector3df_getDistanceFromSQ(handle,pOther.handle)
	EndMethod

	Method dotProduct:Float(pOther:Vector3df)
		Return IrrCore_Vector3df_dotProduct(handle,pOther.handle)
	EndMethod

	Method getLength:Double()
		Return IrrCore_Vector3df_getLength(handle)
	EndMethod

	Method getLengthSQ:Float()
		Return IrrCore_Vector3df_getLengthSQ(handle)
	EndMethod

	Method getArray:Float[]()
		Local retval:Float[4]
		getAs4Values(Varptr(retval[0]))
		Return retval
	EndMethod
	
	Method getAs4Values(vals:Float Ptr)
		IrrCore_Vector3df_getAs4Values(handle,vals)
	EndMethod

	Method getHorizontalAngle:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Vector3df_getHorizontalAngle(handle),True)
	EndMethod

	Method Equals:Int(pOther:Vector3df,tolerance:Float = 0.00005)
		Return IrrCore_Vector3df_Equals(handle,pOther.handle,tolerance)
	EndMethod

	Method getInterpolated_quadratic:Vector3df(v2:Vector3df,v3:Vector3df,d:Float)
		Return Vector3df.createFromHandle(IrrCore_Vector3df_getInterpolated_quadratic(handle,v2.handle,v3.handle,d),True)
	EndMethod

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------
	
	Method setEq:Vector3df(pOther:Vector3df)
		IrrCore_Vector3df_setEq(handle,pOther.handle)
		Return Self
	EndMethod	

	Method EqEq:Int(pOther:Vector3df)
		Return IrrCore_Vector3df_EqEq(handle,pOther.handle)
	EndMethod

	Method NotEq:Int(pOther:Vector3df)
		Return IrrCore_Vector3df_NotEq(handle,pOther.handle)
	EndMethod
	
	Method GreatEq(pOther:Vector3df)
		IrrCore_Vector3df_GreatEq(handle,pOther.handle)
	EndMethod

	Method LessEq(pOther:Vector3df)
		IrrCore_Vector3df_LessEq(handle,pOther.handle)
	EndMethod

	Method Plus:Vector3df(pOther:Vector3df)		
		Return createFromHandle(IrrCore_Vector3df_Plus(handle,pOther.handle))
	EndMethod
	
	Method PlusEq:Vector3df(pOther:Vector3df)
		IrrCore_Vector3df_PlusEq(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method Minus:Vector3df(pOther:Vector3df)		
		Return createFromHandle(IrrCore_Vector3df_Minus(handle,pOther.handle))
	EndMethod
	
	Method MinusEq:Vector3df(pOther:Vector3df)		
		IrrCore_Vector3df_MinusEq(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method MultVect:Vector3df(pOther:Vector3df)
		Return createFromHandle(IrrCore_Vector3df_MultVect(handle,pOther.handle))
	EndMethod
	
	Method Mult:Vector3df(v:Float)
		Return createFromHandle(IrrCore_Vector3df_Mult(handle,v))
	EndMethod

	Method MultEqVect:Vector3df(pOther:Vector3df)
		IrrCore_Vector3df_MultEqVect(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method MultEq:Vector3df(v:Float)
		IrrCore_Vector3df_MultEq(handle,v)
		Return Self
	EndMethod
	
	Method DivVect:Vector3df(pOther:Vector3df)
		Return createFromHandle(IrrCore_Vector3df_DivVect(handle,pOther.handle))
	EndMethod

	Method Div:Vector3df(v:Float)
		Return createFromHandle(IrrCore_Vector3df_Div(handle,v))
	EndMethod

	Method DivEqVect:Vector3df(pOther:Vector3df)
		IrrCore_Vector3df_DivEqVect(handle,pOther.handle)
		Return Self
	EndMethod
	
	Method DivEq:Vector3df(v:Float)
		IrrCore_Vector3df_DivEq(handle,v)
		Return Self
	EndMethod

	Method Less:Int(other:Vector3df)
		Return IrrCore_Vector3df_Less(handle,other.handle)	
	EndMethod
	
	Method Great:Int(other:Vector3df)
		Return IrrCore_Vector3df_Great(handle,other.handle)
	EndMethod
		
	'--------------------------------------------
	' set routines
	'--------------------------------------------

	Method setX(x:Float)
		IrrCore_Vector3df_setX(handle,x)
	EndMethod
	
	Method setY(y:Float)
		IrrCore_Vector3df_setY(handle,y)
	EndMethod
	
	Method setZ(z:Float)
		IrrCore_Vector3df_setZ(handle,z)
	EndMethod

	Method setFromVect(pOther:Vector3df)
		IrrCore_Vector3df_setFromVect(handle,pOther.handle)
	EndMethod

	Method set(x:Float,y:Float,z:Float)
		IrrCore_Vector3df_set(handle,x,y,z)
	EndMethod

	'--------------------------------------------
	' get routines
	'--------------------------------------------

	Method getX:Float()
		Return IrrCore_Vector3df_getX(handle)
	EndMethod
	Method getY:Float()
		Return IrrCore_Vector3df_getY(handle)
	EndMethod
	Method getZ:Float()
		Return IrrCore_Vector3df_getZ(handle)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------

	Function create:Vector3df()
		Return createFromHandle(IrrCore_Vector3df_new())
	EndFunction

	Function createFromVals:Vector3df(x:Float,y:Float,z:Float)
		Return createFromHandle(IrrCore_Vector3df_new(x,y,z))
	EndFunction

	Function createFromVect:Vector3df(vect:Vector3df)
		Return createFromHandle(IrrCore_Vector3df_newFromVect(vect.handle))
	EndFunction

	Function createFromHandle:Vector3df(pVect:Int,bMustDel:Int=True)
		If Not pVect Then Return Null
		Local oRetVal:Vector3df=New Vector3df
		oRetVal.attach(pVect,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Vector3df_del(handle)
	EndMethod


EndType
