' ------------------------------------------------------------------------------------
'  irr::gui::IGUIElement wrappers
'  do Not Delete a pointer To this
' ------------------------------------------------------------------------------------

Public

' enum EGUI_ALIGNMENT
Const EGUIA_UPPERLEFT:Int = 0 
Const EGUIA_LOWERRIGHT:Int = 1
Const EGUIA_CENTER:Int = 2
Const EGUIA_SCALE:Int = 3

' Names for alignments
Global GUIAlignmentNames:String[] = ..
[ ..
	"upperLeft", ..
	"lowerRight", ..
	"center", ..
	"scale", ..
	"" ..
]

Type IGUIElement Extends IAttributeExchangingObject

	Function createFromHandle:IGUIElement(pHandle:Int,bMustDel:Int=False,pGenFunc:Byte Ptr=Null)
		If Not pHandle Then Return Null
		Local oRetVal:IGUIElement=New IGUIElement
		oRetVal.attach(pHandle,bMustDel)
		oRetVal=oRetVal.cast(True)
		Return oRetVal
	EndFunction

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrGUI_IGUIElement_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrGUI_IGUIElement_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrGUI_IGUIElement_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

	' TODO: impelement any new control types
	Method cast:IGUIElement(bTakeOwnership:Int=True)
		Local element_cast:IGUIElement=Null
		Local element_type:Int=getType()
		Local bMustDelete:Int=False

		If bTakeOwnership Then 
			bMustDelete=Self.mustdelete
			Self.mustdelete=False
		EndIf
		
		Select element_type
			Case EGUIET_BUTTON
				element_cast=IGUIButton.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_CHECK_BOX
				element_cast=IGUICheckBox.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_COMBO_BOX
				element_cast=IGUIComboBox.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_CONTEXT_MENU
				element_cast=IGUIContextMenu.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_EDIT_BOX
				element_cast=IGUIEditBox.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_FILE_OPEN_DIALOG
				element_cast=IGUIFileOpenDialog.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_IN_OUT_FADER
				element_cast=IGUIInOutFader.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_IMAGE
				element_cast=IGUIImage.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_LIST_BOX
				element_cast=IGUIListBox.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_MESH_VIEWER
				element_cast=IGUIMeshViewer.createFromHandle(Self.handle,bMustDelete)
			Rem
			Case EGUIET_MODAL_SCREEN
				element_cast=IGUIModalScreen.createFromHandle(Self.handle,bMustDelete)
			EndRem			
			Case EGUIET_SCROLL_BAR
				element_cast=IGUIScrollBar.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_STATIC_TEXT
				element_cast=IGUIStaticText.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_TAB
				element_cast=IGUITab.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_TAB_CONTROL
				element_cast=IGUITabControl.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_TOOL_BAR
				element_cast=IGUIToolBar.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_WINDOW
				element_cast=IGUIWindow.createFromHandle(Self.handle,bMustDelete)
			Case EGUIET_SPIN_BOX
				element_cast=IGUISpinBox.createFromHandle(Self.handle,bMustDelete)
			Default
				element_cast=Self
				Self.mustdelete=bMustDelete
		EndSelect

		Return element_cast
	EndMethod
	
	Method addChild(child:IGUIElement)
		IrrGUI_IGUIElement_addChild(handle,child.handle)
	EndMethod
	
	Method bringToFront:Int(child:IGUIElement)
		Return IrrGUI_IGUIElement_bringToFront:Int(handle,child.handle)
	EndMethod
	
	Method draw()
		IrrGUI_IGUIElement_draw(handle)
	EndMethod 
	
	Method getAbsolutePosition:Recti()
		Return Recti.createFromHandle(IrrGUI_IGUIElement_getAbsolutePosition(handle))
	EndMethod
	
	' dont delete pointer
	Method getElementFromId:IGUIElement(id:Int,bSearchChildren:Int=False)
		Local element:IGUIElement=IGUIElement.createFromHandle(IrrGUI_IGUIElement_getElementFromId(handle,id,bSearchchildren),False)
		If element And element.IsValid() Then element=element.cast()
		Return element
	EndMethod
	
	Method getElementFromPoint:IGUIElement(point:Position2di)
		' dont delete pointer
		Local element:IGUIElement=IGUIElement.createFromHandle(IrrGUI_IGUIElement_getElementFromPoint(handle,point.handle),False)
		If element And element.IsValid() Then element=element.cast()
		Return element
	EndMethod
	
	Method getID:Int()
		Return IrrGUI_IGUIElement_getID(handle)
	EndMethod
	
	Method getParent:IGUIElement()
		' dont delete pointer
		Local element:IGUIElement=IGUIElement.createFromHandle(IrrGUI_IGUIElement_getParent(handle),False)
		If element And element.IsValid() Then element=element.cast()
		Return element
	EndMethod
	
	Method getRelativePosition:Recti()
		Return Recti.createFromHandle(IrrGUI_IGUIElement_getRelativePosition(handle))
	EndMethod
	
	Method getText:String()
		Return WCharToString(IrrGUI_IGUIElement_getText(handle))
	EndMethod
	
	Method isEnabled:Int()
		Return IrrGUI_IGUIElement_isEnabled(handle)
	EndMethod
	
	Method isVisible:Int()
		Return IrrGUI_IGUIElement_isVisible(handle)
	EndMethod
	
	Method move(absoluteMovement:Position2di)
		IrrGUI_IGUIElement_move(handle,absoluteMovement.handle)
	EndMethod
	
	Method remove()
		IrrGUI_IGUIElement_remove(handle)
	EndMethod
	
	Method removeChild(child:IGUIElement)
		IrrGUI_IGUIElement_removeChild(handle,child.handle)
	EndMethod
	
	Method setEnabled(bEnabled:Int)
		IrrGUI_IGUIElement_setEnabled(handle,bEnabled)
	EndMethod 

	Method setID(id:Int)	
		IrrGUI_IGUIElement_setID(handle,id)
	EndMethod
	
	Method setRelativePosition(rpos:Recti)
		IrrGUI_IGUIElement_setRelativePosition(handle,rpos.handle)
	EndMethod
	
	Method setText(text:String)
		IrrGUI_IGUIElement_setText(handle,StringToWChar(text))
	EndMethod
	
	Method setVisible(bVisible:Int)
		IrrGUI_IGUIElement_setVisible(handle,bVisible)
	EndMethod
	
	Method updateAbsolutePosition()
		IrrGUI_IGUIElement_updateAbsolutePosition(handle)
	EndMethod 

	'  note, changed To an array For ease of use
	Method getChildren:IGUIElement[]()
		Local ptrs:Int Ptr	
		Local objs:IGUIElement[]=Null				

		Local size:Int=IrrGUI_IGUIElement_getChildren(handle,Varptr(ptrs))

		If size>0 Then		
			objs=New IGUIElement[size]

			For Local i:Int=0 Until size
				objs[i]=IGUIElement.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs	
	EndMethod
	
	Method getType:Int()
	 	Return IrrGUI_IGUIElement_getType(handle) ' returns EGUI_ELEMENT_TYPE
	EndMethod

	Method setRelativePositionFromFloatRect(rect:Rectf)
		IrrGUI_IGUIElement_setRelativePositionFromFloatRect(handle,rect.handle)
	EndMethod
	
	Method setNotClipped(bNoClip:Int)
		IrrGUI_IGUIElement_setNotClipped(handle,bNoClip)
	EndMethod
	
	Method isNotClipped:Int()
		Return IrrGUI_IGUIElement_isNotClipped(handle)
	EndMethod
	
	Method setMaxSize(size:Dimension2di)
		IrrGUI_IGUIElement_setMaxSize(handle,size.handle)
	EndMethod
	
	Method setMinSize(size:Dimension2di)
		IrrGUI_IGUIElement_setMinSize(handle,size.handle)
	EndMethod
	
	Method setAlignment(EGUI_ALIGNMENT_left:Int,EGUI_ALIGNMENT_right:Int,EGUI_ALIGNMENT_top:Int,EGUI_ALIGNMENT_bottom:Int)
		IrrGUI_IGUIElement_setAlignment(handle,EGUI_ALIGNMENT_left,EGUI_ALIGNMENT_right,EGUI_ALIGNMENT_top,EGUI_ALIGNMENT_bottom)
	EndMethod
	
	Method OnPostRender(timeMs:Int)
		IrrGUI_IGUIElement_OnPostRender(handle,timeMs)
	EndMethod
	
	Method isSubElement:Int()
		Return IrrGUI_IGUIElement_isSubElement(handle)
	EndMethod
	
	Method setSubElement(bSubElement:Int)
		IrrGUI_IGUIElement_setSubElement(handle,bSubElement)
	EndMethod
	
	Method setToolTipText(text:String)
		IrrGUI_IGUIElement_setToolTipText(handle,StringToWChar(text))
	EndMethod
	
	Method getToolTipText:String()
		Return WCharToString(IrrGUI_IGUIElement_getToolTipText(handle))
	EndMethod
	
	Method getTypeName:String()
		Return IrrGUI_IGUIElement_getTypeName(handle)
	EndMethod

	Method getAbsoluteClippingRect:Recti()
		Return Recti.createFromHandle(IrrGUI_IGUIElement_getAbsoluteClippingRect(handle), True)
	EndMethod
	
	Method isPointInside:Int(point:Position2di)
		Return IrrGUI_IGUIElement_isPointInside(handle,point.handle)
	EndMethod
	
	Method setTabStop(bEnable:Int)
		IrrGUI_IGUIElement_setTabStop(handle,bEnable)	
	EndMethod
	
	Method isTabStop:Int()
		Return IrrGUI_IGUIElement_isTabStop(handle)
	EndMethod
	
	Method setTabOrder(index:Int)
		IrrGUI_IGUIElement_setTabOrder(handle,index)
	EndMethod
	
	Method getTabOrder:Int()
		Return IrrGUI_IGUIElement_getTabOrder(handle)
	EndMethod
	
	Method setTabGroup(bIsGroup:Int)
		IrrGUI_IGUIElement_setTabGroup(handle,bIsGroup)	
	EndMethod
	
	Method isTabGroup:Int()
		Return IrrGUI_IGUIElement_isTabGroup(handle)	
	EndMethod
	
	Method getTabGroup:IGUIElement()
		Return IGUIElement.createFromHandle(IrrGUI_IGUIElement_getTabGroup(handle), False)
	EndMethod
	
	Method isMyChild:Int(child:IGUIElement)
		Return IrrGUI_IGUIElement_isMyChild(handle,child.handle) 
	EndMethod
	
	' NOTE: first and closest should be IGUIElement variables with a value of null
	Method getNextElement:Int(startOrder:Int, bReverse:Int, bGroup:Int, first:IGUIElement Var, closest:IGUIElement Var, bIncludeInvisible:Int = False)
		Local first_ptr:Int
		Local closest_ptr:Int
	
		Local bFound:Int = IrrGUI_IGUIElement_getNextElement:Int(handle,startOrder,bReverse,bGroup,Varptr(first_ptr),Varptr(closest_ptr),bIncludeInvisible)

		first = IGUIElement.createFromHandle(first_ptr, False)
		closest = IGUIElement.createFromHandle(closest_ptr, False)
		
		Return bFound
	EndMethod

EndType


